#!/usr/bin/env bash
#
# Packer shell provisioner for Arch Linux on Raspberry Pi
#
# set -o errexit
# set -o nounset
set -o xtrace

# Set hostname
echo "${HOSTNAME}" > /etc/hostname

# Resolve hostname
cat << EOF > /etc/hosts
127.0.0.1        localhost
::1              localhost ip6-localhost ip6-loopback
ff02::1          ip6-allnodes
ff02::2          ip6-allrouters

127.0.1.1        $HOSTNAME
EOF

# Install script to resize fs
mv /tmp/resizerootfs.service /etc/systemd/system/
chmod +x /tmp/resizerootfs
mv /tmp/resizerootfs /usr/sbin/
systemctl enable resizerootfs.service

# Add HashiCorp repository
wget -O- https://apt.releases.hashicorp.com/gpg \
 | gpg --dearmor -o /usr/share/keyrings/hashicorp-archive-keyring.gpg
echo "deb [signed-by=/usr/share/keyrings/hashicorp-archive-keyring.gpg] https://apt.releases.hashicorp.com $(lsb_release -cs) main" \
 | tee /etc/apt/sources.list.d/hashicorp.list

# Install packages
apt-get update
DEBIAN_FRONTEND=noninteractive apt-get install -y jq podman cloud-init \
 "nomad=${NOMAD_VERSION}-1"

curl -L -o openbao.deb "https://github.com/openbao/openbao/releases/download/v${BAO_VERSION}/bao_${BAO_VERSION}_linux_arm64.deb"
apt install ./openbao.deb

# Configure insecure local registry
cat << EOF > /etc/containers/registries.conf
unqualified-search-registries = ['127.0.0.1:5000', 'haproxy.lan:5000', 'docker.io']

[[registry]]
location = "127.0.0.1:5000"
insecure = true
[[registry]]
location = "haproxy.lan:5000"
insecure = true
EOF

# Set up no-password sudo
rm /etc/sudoers.d/010_pi-nopasswd
echo '%sudo ALL=(ALL) NOPASSWD: ALL' > /etc/sudoers.d/010_all-nopasswd

# Enable ssh and disable password auth
touch /boot/ssh
sed -i 's/#PasswordAuthentication yes/PasswordAuthentication no/g' /etc/ssh/sshd_config

# Delete default user pi
userdel -r pi

# Create user
useradd -m "${USERNAME}"
usermod -aG sudo "${USERNAME}"

# Setup ssh keys
mkdir "/home/${USERNAME}/.ssh"
touch "/home/${USERNAME}/.ssh/authorized_keys"
cat << EOF > "/home/${USERNAME}/.ssh/authorized_keys"
$AUTHORIZED_KEYS
EOF

chown -R $USERNAME "/home/${USERNAME}/.ssh"
chmod 700 "/home/${USERNAME}/.ssh"
chmod 600 "/home/${USERNAME}/.ssh/authorized_keys"
