#!/usr/bin/env bash
#
# Creates a set of certificates for use with HashiCorp Nomad
# https://developer.hashicorp.com/nomad/tutorials/transport-security/security-enable-tls

# set -o errexit
# set -o nounset
# set -o xtrace

# https://www.shellhacks.com/yes-no-bash-script-prompt-confirmation/
read -p "Do you want to generate a new set of Nomad certicates in the directory \"./tls/nomad/\" [y/N]? " -n 1 -r
echo    # (optional) move to a new line
if [[ ! $REPLY =~ ^[Yy]$ ]]
then
    exit 1
fi

# Set working dir
NOMAD_TLS_BASE_PATH="${NOMAD_TLS_BASE_PATH:-./tls/nomad/}"
mkdir -p "$NOMAD_TLS_BASE_PATH"
cd "$NOMAD_TLS_BASE_PATH"

# Cleanup previously generated certificates
rm -rf certs

# Define cert types
crt_types=("server" "cli" "client")

for type in "${crt_types[@]}"; do
  # Create certificate
  nomad tls cert create -region dc1 -days 3560 \
	  -additional-ipaddress 10.0.0.100 \
	  -additional-ipaddress 10.0.0.101 \
	  -additional-ipaddress 10.0.0.102 \
	  -additional-ipaddress 10.0.0.103 \
	  -additional-ipaddress 10.0.0.104 \
	  -${type}
  cert="dc1-$type-nomad.pem"
  key="dc1-$type-nomad-key.pem"

  # Show fingerprint
  openssl x509 -in $cert -fingerprint -noout
done;

# Move to certs folder
mkdir certs
mv dc1-* certs/
